﻿//Copyright (c) Microsoft Corporation.  All rights reserved.

// The following code was generated by Microsoft Visual Studio 2005.
// The test owner should check each test for validity.
using Microsoft.VisualStudio.TestTools.UnitTesting;
using System;
using System.Text;
using System.Collections.Generic;
using Eas.WebCrawler.Interfaces;
using System.Net;
using System.IO;
using Eas.WebCrawler.Model.Core;
using Eas.WebCrawler.Model.Actions;

namespace Eas.WebCrawler.Test.Model {
    /// <summary>
    ///This is a test class for Eas.WebCrawler.Model.BaseProcessingState and is intended
    ///to contain all Eas.WebCrawler.Model.BaseProcessingState Unit Tests
    ///</summary>
    [TestClass()]
    public class BaseProcessingStateTest {


        private TestContext testContextInstance;

        /// <summary>
        ///Gets or sets the test context which provides
        ///information about and functionality for the current test run.
        ///</summary>
        public TestContext TestContext {
            get {
                return testContextInstance;
            }
            set {
                testContextInstance = value;
            }
        }
        #region Additional test attributes
        // 
        //You can use the following additional attributes as you write your tests:
        //
        //Use ClassInitialize to run code before running the first test in the class
        //
        //[ClassInitialize()]
        //public static void MyClassInitialize(TestContext testContext)
        //{
        //}
        //
        //Use ClassCleanup to run code after all tests in a class have run
        //
        //[ClassCleanup()]
        //public static void MyClassCleanup()
        //{
        //}
        //
        //Use TestInitialize to run code before running each test
        //
        //[TestInitialize()]
        //public void MyTestInitialize()
        //{
        //}
        //
        //Use TestCleanup to run code after each test has run
        //
        [TestCleanup()]
        public void MyTestCleanup() {
            GC.Collect();
        }
        //
        #endregion

        [TestMethod]
        /// <summary>Executes the TestSimplePropeties test.</summary>
        public void TestSimplePropeties() {
            int id = 32; // TODO: Initialize to an appropriate value
            using (NMock2.Mockery mockery = new NMock2.Mockery()) {
                IAsyncResult mockResult = mockery.NewMock<IAsyncResult>();
                WebResponseInterface webResponse = mockery.NewMock<WebResponseInterface>();
                WebRequestInterface webRequest = mockery.NewMock<WebRequestInterface>();
                WorkerContext workerContext = null;
                TemporaryStorage temporaryStorage = mockery.NewMock<TemporaryStorage>();




                using (BaseProcessingState state = new BaseProcessingState(workerContext, id)) {
                    Assert.AreEqual<WorkerState>(WorkerState.Unstarted, state.WorkerState);
                    Assert.AreSame(Eas.WebCrawler.Model.Actions.UnstartedAction.Instance, state.Action);
                    Assert.IsFalse(state.WasPaused);
                    Assert.IsNull(state.WebRequest);
                    Assert.IsNull(state.WebResponse);
                    Assert.IsNull(state.ResponseStream);
                    Assert.IsNull(state.Request);
                    Assert.IsTrue(state.IsTextResponse);
                    Assert.AreEqual<int>(id, state.Id);
//                    Assert.IsFalse(state.WasTerminated);
                    Assert.IsNull(state.PausedAction);
                    Assert.IsNull(state.PausedResult);
                    Assert.AreEqual<int>(0, state.RedirectDepth);
                    Assert.IsFalse(state.StreamReadHasCompleted);

                    TestAction stoppedAction = new TestAction(ActionId.Terminal);

                    state.PausedAction = stoppedAction;
                    Assert.AreSame(stoppedAction, state.PausedAction);
                    state.PausedResult = mockResult;
                    Assert.AreSame(mockResult, state.PausedResult);
                    state.RedirectDepth = 32;
                    Assert.AreEqual<int>(32, state.RedirectDepth);
                    state.StreamReadHasCompleted = true;
                    Assert.IsTrue(state.StreamReadHasCompleted);


                    state.IsTextResponse = false;
                    Assert.IsFalse(state.IsTextResponse);
                    Request request = RequestHelper.rHtmlRef;
                    state.Request = request;
                    Assert.AreSame(request, state.Request);
                    Stream stream = new AllExceptionStream();
                    state.ResponseStream = stream;
                    Assert.AreSame(stream, state.ResponseStream);
                    Assert.IsFalse(state.WasPaused);
                    state.SetPaused();
                    Assert.IsTrue(state.WasPaused);
                    state.WebRequest = webRequest;
                    Assert.IsNotNull(webRequest);
                    Assert.AreSame(webRequest, state.WebRequest);
                    state.WebResponse = webResponse;
                    Assert.IsNotNull(webResponse);
                    Assert.AreSame(webResponse, state.WebResponse);
                    state.ResponseStore = temporaryStorage;
                    Assert.AreSame(temporaryStorage, state.ResponseStore);
                    Assert.AreEqual<int>(BaseProcessingState.ResponseBufferSize, state.ResponseBuffer.Length);
                    Eas.WebCrawler.Model.StorageWriters.TextStorageWriter storageWriter = new Eas.WebCrawler.Model.StorageWriters.TextStorageWriter();
                    state.StorageWriter = storageWriter;
                    Assert.AreSame(storageWriter, state.StorageWriter);

                    //Abort request
                    state.AbortRequest();
//                    Assert.IsTrue(state.WasTerminated);

                    //Test that AbortRequest did not clear out state data...
                    Assert.AreSame(storageWriter, state.StorageWriter);
                    Assert.AreEqual<int>(BaseProcessingState.ResponseBufferSize, state.ResponseBuffer.Length);
                    Assert.AreSame(temporaryStorage, state.ResponseStore);
                    Assert.IsFalse(state.IsTextResponse);
                    Assert.AreSame(request, state.Request);
                    Assert.AreSame(stream, state.ResponseStream);
                    Assert.IsTrue(state.WasPaused);
                    Assert.IsNotNull(webRequest);
                    Assert.AreSame(webRequest, state.WebRequest);
                    Assert.IsNotNull(webResponse);
                    Assert.AreSame(webResponse, state.WebResponse);

                    state.Reset();

                    //Test that Reset() did clear out data.
                    Assert.IsNull(state.StorageWriter);
                    Assert.AreEqual<int>(BaseProcessingState.ResponseBufferSize, state.ResponseBuffer.Length);
                    Assert.IsNull(state.ResponseStore);
                    Assert.IsFalse(state.StreamReadHasCompleted);
                    Assert.AreEqual<int>(0, state.RedirectDepth);
                    Assert.IsNull(state.PausedResult);
                    Assert.IsNull(state.PausedAction);
                    //Assert.IsFalse(state.WasTerminated);
                    Assert.IsFalse(state.WasPaused);
                    Assert.IsNull(state.WebRequest);
                    Assert.IsNull(state.WebResponse);
                    Assert.IsNull(state.ResponseStream);
                    Assert.IsNull(state.Request);
                    Assert.IsTrue(state.IsTextResponse);
                    Assert.AreEqual<int>(id, state.Id);
                    state.Reset();

                    //Test that we can call Reset twice and that everything is ok.
                    Assert.IsFalse(state.StreamReadHasCompleted);
                    Assert.AreEqual<int>(0, state.RedirectDepth);
                    Assert.IsNull(state.PausedResult);
                    Assert.IsNull(state.PausedAction);
                    //Assert.IsFalse(state.WasTerminated);
                    Assert.IsFalse(state.WasPaused);
                    Assert.IsNull(state.WebRequest);
                    Assert.IsNull(state.WebResponse);
                    Assert.IsNull(state.ResponseStream);
                    Assert.IsNull(state.Request);
                    Assert.IsTrue(state.IsTextResponse);
                    Assert.AreEqual<int>(id, state.Id);

                }
            }
        }

        [TestMethod]
        /// <summary>Executes the TestChangeRequestManagement test.</summary>
        public void TestChangeRequestManagement() {
            using (BaseProcessingState state = new BaseProcessingState(null, 1)) {
                VerifyEmpty(state);
                state.RequestChange(WorkerState.Working);
                state.RequestChange(WorkerState.Paused);
                state.RequestChange(WorkerState.Working);
                state.RequestChange(WorkerState.Stopped);
                VerifyAllValues(state, WorkerState.Working, WorkerState.Paused, WorkerState.Working, WorkerState.Stopped);
                state.RequestChange(WorkerState.Working);
                state.RequestChange(WorkerState.Paused);
                state.RequestChange(WorkerState.Working);
                state.RequestChange(WorkerState.Stopped);
                using (BaseProcessingState newState = new BaseProcessingState(null, 2)) {
                    state.CopyChangeRequests(newState);
                    VerifyEmpty(state);
                    VerifyAllValues(newState, WorkerState.Working, WorkerState.Paused, WorkerState.Working, WorkerState.Stopped);
                }
            }
        }

        private void VerifyAllValues(BaseProcessingState state, params WorkerState[] values) {
            foreach (WorkerState val in values) {
                VerifyValue(state, val);
            }
            VerifyEmpty(state);
        }

        private void VerifyEmpty(BaseProcessingState state) {
            Eas.WebCrawler.Interfaces.WorkerState? result = state.DeQueueChangeRequest();
            Assert.IsFalse(result.HasValue);
        }

        private void VerifyValue(BaseProcessingState state, WorkerState workerState) {
            WorkerState? result = state.DeQueueChangeRequest();
            Assert.IsTrue(result.HasValue);
            Assert.AreEqual<WorkerState>(workerState, result.Value);
        }

        /// <summary>
        ///A test for BaseProcessingState (int)
        ///</summary>
        [TestMethod()]
        public void ConstructorTest() {
            int id = 32; // TODO: Initialize to an appropriate value

            using (BaseProcessingState state = new BaseProcessingState(null, id)) {
                Assert.AreEqual<int>(id, state.Id);
            }
        }
    }


}
